#ifndef __USER_INTERFACE_H
#define __USER_INTERFACE_H



typedef signed char      S8;
typedef unsigned char    U8;
typedef signed short     S16;
typedef unsigned short   U16;
typedef signed long      S32;
typedef unsigned long    U32;
typedef signed _int64    S64;
typedef unsigned _int64  U64;



/**********************************************
*               Definitions
**********************************************/
#define K_3078_DLL_VERSION       100

#define K_3078_MAX_DEVICE        12

#define K_USED_BAR_MEM    6         // PCI_NUM_BARS_TYPE_00 in Plx.h

// Aufteilung 12 Byte Header
#define K_HEADER_POS_START_BYTE         0
#define K_HEADER_POS_FLAGS              1
#define K_HEADER_POS_LENGTH_LOW         2
#define K_HEADER_POS_LENGTH_HIGH        3
#define K_HEADER_POS_TARGET_ADDRESS     4  // bei PXI Portnummer
#define K_HEADER_POS_TARGET_PORT        5
#define K_HEADER_POS_SOURCE_ADDRESS     6
#define K_HEADER_POS_SOURCE_PORT        7
#define K_HEADER_POS_TYPE               8
#define K_HEADER_POS_APPLICATIONHANDLE  9
#define K_HEADER_POS_DEVICE            10  // bei PXI Kartennummer
#define K_HEADER_POS_COMMAND           11
#define K_HEADER_POS_FIRST_DATA        12

#define K_NUMBER_COUNT_HEADER          12

// ---------- XILINX Write ---------------------------------------------------------
// XILINX command structure
#define K_HEADER_XILINX_INSTRUCTION_GLOBAL      0    // start byte
#define K_HEADER_XILINX_CARD_NUMBER             1    // device (1 ... K_DEV_MAX)
#define K_HEADER_XILINX_PORT_NUMBER             2    // not used
#define K_HEADER_XILINX_INSTRUCTION             3    // command
#define K_HEADER_XILINX_FIRST_DATA              4    
#define K_HEADER_XILINX_LENGTH                  4     

#define K_READ                      0
#define K_WRITE                     1

#define K_HEADER_XILINX_START_BYTE  0x48

// XILINX commands
#define K_XILINX_CMD_PO_RESET       0x10
#define K_XILINX_CMD_EXT_CLOCK      0x11
#define K_XILINX_CMD_REG_WRITE      0x12

// ---------- interrupt ----------------------------------------------------------
// interrupt mode - PLX
#define K_INTERRUPT_OFF							0x00
#define K_INTERRUPT_ON							0x01

// interrupt identifiers - bit coding - PLX
#define K_INTR_TYPE_NONE						0x00
#define K_INTR_TYPE_READ_DATA					0x01
//#define K_INTR_TYPE_ ...						0x02
//#define K_INTR_TYPE_ ...						0x04
//#define K_INTR_TYPE_ ...						0x08

// interrupt handling mode - FPGA
#define K_FPGA_FIFO_MODE_POLLING				0x00
#define K_FPGA_FIFO_MODE_INTR					0x01
#define K_FPGA_FIFO_MODE_DMA					0x02

#define K_FPGA_INTR_ACK_INACTIV					0x00000008
#define K_FPGA_INTR_TIMER_INACTIV				0x00003FF0

// interrupt source register - FPGA
#define K_XILINX_MAIN_INTR_SOURCE_READ_1		0x00000001
#define K_XILINX_MAIN_INTR_SOURCE_MONITOR_1		0x00000002
#define K_XILINX_MAIN_INTR_SOURCE_READ_2		0x00000008
#define K_XILINX_MAIN_INTR_SOURCE_MONITOR_2		0x00000010

/**********************************************
*               Structures
**********************************************/
typedef struct
	{
	U32		SW_Version_XILINX;
	U32		HW_Version_XILINX;
	U32		HW_Name_XILINX;
	U32		Interface_XILINX;
	U32		GeoAddress;
	U32		SerialNumber;
	} t_Card_Info;

typedef struct
	{
	U32				DllVersion;
	U32				DriverVersion;
	U32				CardCount;
	t_Card_Info		CardInfo[K_3078_MAX_DEVICE];
	} t_Driver_Info;

typedef struct
	{
	U32		PciBarBaseAddress[K_USED_BAR_MEM];
	U32		PciBarRange[K_USED_BAR_MEM];
	U32		MemoryType[K_USED_BAR_MEM];                     // 0 -> IO ; 1 -> Memory
	} t_MemProperties;

typedef struct
	{
	U32		Mode;
	U32		Register;
	U32		Value;
	} t_RegReadWrite;

typedef struct _tCmdPackHeader
	{
	U32		mSubCode;                       // LSBUS_CMD_PACKAGE
	S32		mSerial;                        // -1: erstbester
	U32		mCountCmds;                     // total number of commands
	U32		mSizeCmds;                      // total number of bytes following command data
	}TCmdPackHeader, *PCmdPackHeader;

typedef struct
	{ 
	U32		Device;
	U32		EventTimeout_ms;
	U32		DeviceMemory;
	HANDLE	Handle;
	} t_InterruptHandling;

typedef struct
	{ 
	U16		ReadFifo_Mode;
	U16		ReadFifo_TimeValue;
	U16		ReadFifo_DataThreshold;
	U16		Monitor_Mode;
	U16		Monitor_TimeValue;
	U16		Monitor_DataThreshold;
	} t_InterruptHandling_FPGA;

typedef struct
{
    unsigned int InboundPost      :1;
    unsigned int OutboundPost     :1;
    unsigned int OutboundOverflow :1;
    unsigned int OutboundOption   :1;
    unsigned int IopDmaChannel0   :1;
    unsigned int PciDmaChannel0   :1;
    unsigned int IopDmaChannel1   :1;
    unsigned int PciDmaChannel1   :1;
    unsigned int IopDmaChannel2   :1;
    unsigned int PciDmaChannel2   :1;
    unsigned int Mailbox0         :1;
    unsigned int Mailbox1         :1;
    unsigned int Mailbox2         :1;
    unsigned int Mailbox3         :1;
    unsigned int Mailbox4         :1;
    unsigned int Mailbox5         :1;
    unsigned int Mailbox6         :1;
    unsigned int Mailbox7         :1;
    unsigned int IopDoorbell      :1;
    unsigned int PciDoorbell      :1;
    unsigned int BIST             :1;
    unsigned int PowerManagement  :1;
    unsigned int PciMainInt       :1;
    unsigned int IopToPciInt      :1;
    unsigned int IopMainInt       :1;
    unsigned int PciAbort         :1;
    unsigned int PciReset         :1;
    unsigned int PciPME           :1;
    unsigned int Enum             :1;
    unsigned int AbortLSERR       :1;
    unsigned int ParityLSERR      :1;
    unsigned int RetryAbort       :1;
    unsigned int LocalParityLSERR :1;
    unsigned int PciSERR          :1;
    unsigned int IopToPciInt_2    :1;
    unsigned int Message          :1;
    unsigned int SwInterrupt      :1;
    unsigned int SecResetDeassert :1;
    unsigned int SecPmeDeassert   :1;
    unsigned int GPIO14           :1;
    unsigned int GPIO4            :1;
} t_Interrupt_PLX;

typedef struct
{
    U32 IsValidTag;
	U32	Reserved;
    U64 pWaitObject;
    U64 hEvent;
} t_Notify_Object_PLX;



typedef struct
 { 
 U8 Device;                 // Device Nummer im Handle Array 
 U8 Space;                  // Space 0/1 fr Mikrocontroller 1/2
 U32 TargetAddr;            // Anfangsaddresse des ZielPuffers
 U32 TargetBufferLength;    // Gre des ZielPuffers    
 } DMA_setup, *pDMA_setup;

/******************************************
 * Goepel function
 *****************************************/
#ifdef __EXPORT_DLL	// creating DLL
	#define INTEGRATION_MODE	__declspec(dllexport)
#else				// using in customer application
	#define INTEGRATION_MODE	__declspec(dllimport)
#endif

#ifdef __cplusplus
	extern "C" {
#endif


INTEGRATION_MODE		S32
						Pxi3078__DriverInfo(
												t_Driver_Info		*pDriverInfo,
												U32					Length);
INTEGRATION_MODE		S32
						Pxi3078__XilinxReadWriteRegister(
												U8					*pData, 
												U32					*DataLength);
INTEGRATION_MODE		S32
						Pxi3078__WriteInstruction(
												U8					*pData, 
												U16					DataLength);
INTEGRATION_MODE		S32
						Pxi3078__ReadResponse(
												U8					Device,
												U8					*pData, 
												U32					*DataLength);
INTEGRATION_MODE		S32
						Pxi3078__FPGA_Reconfiguration(
												U8					Device,
												S8					*pFileData);

// interrupt support function
INTEGRATION_MODE		S32	Pxi3078__SetInterruptSource(
												U8		Device,
												U8		ResponseMode, 
												U8		InterruptMode, 
												U8		InterruptType);
INTEGRATION_MODE		S32	Pxi3078__SetInterruptHandling(
												U8							Device,
												U16*						InterruptHandling);
//												t_InterruptHandling_FPGA	IntrHandling);
INTEGRATION_MODE		S32	Pxi3078__NotificationInit(
												HANDLE				hDevice,
												t_Interrupt_PLX		*pInterrupt,
												t_Notify_Object_PLX	*pEvent);
INTEGRATION_MODE		S32	Pxi3078__NotificationWait(
												HANDLE				hDevice,
												t_Notify_Object_PLX	*pEvent,
												U32					Timeout_ms);
INTEGRATION_MODE		S32	Pxi3078__NotificationCancel(
												HANDLE				hDevice,
												t_Notify_Object_PLX	*pEvent);






#ifdef __cplusplus
	}
#endif


#endif
